"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControlUnit = exports.RemoteKey = void 0;
const callbackify_1 = require("../util/callbackify");
/**
 * Enum describing remote key presses from homebridge.
 */
var RemoteKey;
(function (RemoteKey) {
    RemoteKey[RemoteKey["REWIND"] = 0] = "REWIND";
    RemoteKey[RemoteKey["FAST_FORWARD"] = 1] = "FAST_FORWARD";
    RemoteKey[RemoteKey["NEXT_TRACK"] = 2] = "NEXT_TRACK";
    RemoteKey[RemoteKey["PREVIOUS_TRACK"] = 3] = "PREVIOUS_TRACK";
    RemoteKey[RemoteKey["ARROW_UP"] = 4] = "ARROW_UP";
    RemoteKey[RemoteKey["ARROW_DOWN"] = 5] = "ARROW_DOWN";
    RemoteKey[RemoteKey["ARROW_LEFT"] = 6] = "ARROW_LEFT";
    RemoteKey[RemoteKey["ARROW_RIGHT"] = 7] = "ARROW_RIGHT";
    RemoteKey[RemoteKey["SELECT"] = 8] = "SELECT";
    RemoteKey[RemoteKey["BACK"] = 9] = "BACK";
    RemoteKey[RemoteKey["EXIT"] = 10] = "EXIT";
    RemoteKey[RemoteKey["PLAY_PAUSE"] = 11] = "PLAY_PAUSE";
    RemoteKey[RemoteKey["INFORMATION"] = 15] = "INFORMATION";
})(RemoteKey = exports.RemoteKey || (exports.RemoteKey = {}));
class ControlUnit {
    constructor(_platform, _accessory, _activityService, _commandService, _volumeService, _activities) {
        this._platform = _platform;
        this._accessory = _accessory;
        this._activityService = _activityService;
        this._commandService = _commandService;
        this._volumeService = _volumeService;
        this._activities = _activities;
        /**
         * Event handler for SET active characteristic
         */
        this.onSetAccessoryActive = (value) => __awaiter(this, void 0, void 0, function* () {
            switch (value) {
                case 0:
                    this._activityService.stopCurrentActivity(this._accessory.displayName);
                    break;
                //Turn on with first activity
                case 1:
                    if (this._activityService.getIsActive(this._accessory.displayName)) {
                        break;
                    }
                    this._activityService.startLastOrDefaultActivity(this._accessory.displayName, this._activities[0]);
                    break;
            }
        });
        /**
         * Event handler for GET active characteristic
         */
        this.onGetAccessoryActive = () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            //@ts-ignore
            return ((_a = this._activityService.getCurrentActivity(this._accessory.displayName)) === null || _a === void 0 ? void 0 : _a.DisplayName) ? this._platform.Characteristic.Active.ACTIVE
                : this._platform.Characteristic.Active.INACTIVE;
        });
        /**
         * Event handler for SET remote key
         */
        this.onSetRemoteKey = (key) => __awaiter(this, void 0, void 0, function* () {
            this._commandService.sendKeyPress(this._accessory.displayName, key);
        });
        /**
         * Event handler for SET active identifier characteristic
         */
        this.onSetActiveIdentifier = (identifier) => __awaiter(this, void 0, void 0, function* () {
            this._activityService.startActivity(this._accessory.displayName, this._activities[identifier]);
        });
        /**
         * Event handler for GET active identifier characteristic
         */
        this.onGetActiveIdentifier = () => __awaiter(this, void 0, void 0, function* () {
            let currentActivity = this._activityService.getCurrentActivity(this._accessory.displayName);
            let identifier = 0;
            if (currentActivity) {
                identifier = this._activities.findIndex((e) => e.DisplayName === currentActivity.DisplayName);
            }
            return identifier;
        });
        /**
         * Event handler for SET volume characteristic
         */
        this.onSetVolumeSelector = (value) => __awaiter(this, void 0, void 0, function* () {
            switch (value) {
                case 0:
                    this._volumeService.volumeUp(this._accessory.displayName);
                    break;
                case 1:
                    this._volumeService.volumeDown(this._accessory.displayName);
                    break;
            }
        });
        this._accessory
            .getService(this._platform.Service.AccessoryInformation)
            .setCharacteristic(this._platform.Characteristic.Manufacturer, "Brandon Watson")
            .setCharacteristic(this._platform.Characteristic.Model, "Matrix Output Television")
            .setCharacteristic(this._platform.Characteristic.SerialNumber, "123-456-789");
        const televisionUUID = this._platform.api.hap.uuid.generate(`${this._accessory.displayName} Television`);
        //Init services
        const televisionService = this._accessory.getService(this._platform.Service.Television) ||
            this._accessory.addService(this._platform.Service.Television, "Television", televisionUUID);
        this.configureTvService(televisionService);
        const televisionSpeakerUUID = this._platform.api.hap.uuid.generate(`${this._accessory.displayName} Television Speaker`);
        const televisionSpeakerService = this._accessory.getService(this._platform.Service.TelevisionSpeaker) ||
            this._accessory.addService(this._platform.Service.TelevisionSpeaker, "Television Speaker", televisionSpeakerUUID);
        this.configureTvSpeakerService(televisionService, televisionSpeakerService);
        this.configureInputSourceService(televisionService);
    }
    /*************
     *
     * Tv Service
     *
     *************/
    /**
     * Configure television service
     */
    configureTvService(televisionService) {
        televisionService.setCharacteristic(this._platform.Characteristic.ConfiguredName, this._accessory.displayName);
        //@ts-ignore
        televisionService.setCharacteristic(this._platform.Characteristic.SleepDiscoveryMode, this._platform.Characteristic.SleepDiscoveryMode.ALWAYS_DISCOVERABLE);
        televisionService.setCharacteristic(this._platform.Characteristic.ActiveIdentifier, 1);
        televisionService.setCharacteristic(this._platform.Characteristic.Active, false);
        //setup listeners
        televisionService
            .getCharacteristic(this._platform.Characteristic.Active)
            //@ts-ignore
            .on("set", callbackify_1.default(this.onSetAccessoryActive))
            //@ts-ignore
            .on("get", callbackify_1.default(this.onGetAccessoryActive));
        //Set remote characteristics if is external
        televisionService
            .getCharacteristic(this._platform.Characteristic.RemoteKey)
            //@ts-ignore
            .on("set", callbackify_1.default(this.onSetRemoteKey));
        televisionService
            .getCharacteristic(this._platform.Characteristic.ActiveIdentifier)
            //@ts-ignore
            .on("set", callbackify_1.default(this.onSetActiveIdentifier))
            //@ts-ignore
            .on("get", callbackify_1.default(this.onGetActiveIdentifier));
    }
    /******************
     *
     * Speaker Service
     *
     *****************/
    /**
     * Configure Speaker Service
     */
    configureTvSpeakerService(televisionService, televisionSpeakerService) {
        televisionSpeakerService.setCharacteristic(this._platform.Characteristic.Name, this._accessory.displayName);
        //@ts-ignore
        televisionSpeakerService.setCharacteristic(this._platform.Characteristic.Active, this._platform.Characteristic.Active.ACTIVE);
        //@ts-ignore
        televisionSpeakerService.setCharacteristic(this._platform.Characteristic.VolumeControlType, this._platform.Characteristic.VolumeControlType.ABSOLUTE);
        televisionSpeakerService.subtype = this._accessory.displayName + "Volume";
        if (televisionService) {
            //@ts-ignore
            televisionService.addLinkedService(televisionSpeakerService);
        }
        //Setup listeners
        televisionSpeakerService
            .getCharacteristic(this._platform.Characteristic.VolumeSelector)
            //@ts-ignore
            .on("set", callbackify_1.default(this.onSetVolumeSelector));
    }
    /*****************
     *
     * Input services
     *
     *****************/
    /**
     * Configure input service
     */
    configureInputSourceService(televisionService) {
        let inputs = [];
        this._activities.forEach((activity, index) => {
            const inputUUID = this._platform.api.hap.uuid.generate(`${activity.DisplayName} Input`);
            let inputService = this._accessory.getService(activity.DisplayName) ||
                this._accessory.addService(this._platform.Service.InputSource, activity.DisplayName, inputUUID);
            inputService
                .setCharacteristic(this._platform.Characteristic.Identifier, index)
                .setCharacteristic(this._platform.Characteristic.ConfiguredName, activity.DisplayName)
                .setCharacteristic(this._platform.Characteristic.IsConfigured, 
            //@ts-ignore
            this._platform.Characteristic.IsConfigured.CONFIGURED)
                //@ts-ignore
                .setCharacteristic(this._platform.Characteristic.InputSourceType, this._platform.Characteristic.InputSourceType.HDMI);
            //@ts-ignore
            televisionService.addLinkedService(inputService);
            inputs.push(inputService);
        });
    }
}
exports.ControlUnit = ControlUnit;
//# sourceMappingURL=controlUnit.js.map