"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceButton = void 0;
class DeviceButton {
    constructor(_platform, _accessory, _dataProvider, _deviceInfo) {
        this._platform = _platform;
        this._accessory = _accessory;
        this._dataProvider = _dataProvider;
        this._deviceInfo = _deviceInfo;
        /**
         * Handler for switch set event
         * @param callback The callback function to call when complete
         */
        this.onSwitchSet = (newState, callback) => __awaiter(this, void 0, void 0, function* () {
            if (!this._deviceInfo.IsStateful && newState === this._buttonState) {
                return callback();
            }
            //Get device command if we don't have it
            if (!this._device) {
                this._device = this._dataProvider.getDeviceByName(this._deviceInfo.DeviceName);
            }
            //Execute command
            if (!this._device) {
                return callback();
            }
            //change state if stateful
            if (this._deviceInfo.IsStateful && this._buttonState != newState) {
                this._buttonState = newState;
                yield this._dataProvider.sendCommand(this._deviceInfo.ButtonName, this._device);
                return callback();
            }
            else if (!this._deviceInfo.IsStateful) {
                //Send the number of configured key presses
                for (let i = 0; i < this._deviceInfo.NumberOfKeyPresses; i++) {
                    yield this._dataProvider.sendCommand(this._deviceInfo.ButtonName, this._device);
                }
                this._switchService
                    .getCharacteristic(this._platform.Characteristic.On)
                    .updateValue(false);
                this._buttonState = false;
                return callback();
            }
        });
        /**
         * Handler for switch get event
         * @param callback The callback function to call when complete
         */
        this.onSwitchGet = (callback) => {
            //Only return state if button is stateful
            if (this._deviceInfo.IsStateful) {
                return callback(null, this._buttonState);
            }
            else {
                return callback(null, false);
            }
        };
        this._buttonState = false;
        if (this._deviceInfo.NumberOfKeyPresses && this._deviceInfo.IsStateful) {
            throw new Error("A button cannot be stateful and be pressed more than once");
        }
        this._accessory
            .getService(this._platform.Service.AccessoryInformation)
            .setCharacteristic(this._platform.Characteristic.Manufacturer, "Brandon Watson")
            .setCharacteristic(this._platform.Characteristic.Model, "Device Button")
            .setCharacteristic(this._platform.Characteristic.SerialNumber, "123-456-789");
        const switchUUID = this._platform.api.hap.uuid.generate(`${this._accessory.displayName} Switch`);
        this._switchService =
            this._accessory.getService(this._platform.Service.Switch) ||
                this._accessory.addService(this._platform.Service.Switch, this._accessory.displayName, switchUUID);
        this._switchService
            .getCharacteristic(this._platform.Characteristic.On)
            //@ts-ignore
            .on("set", this.onSwitchSet)
            .updateValue(this._buttonState)
            .on("get", this.onSwitchGet);
    }
}
exports.DeviceButton = DeviceButton;
//# sourceMappingURL=deviceButton.js.map