"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sequence = void 0;
const util_1 = require("../util");
class Sequence {
    constructor(_platform, _accessory, _dataProvider, _sequence) {
        this._platform = _platform;
        this._accessory = _accessory;
        this._dataProvider = _dataProvider;
        this._sequence = _sequence;
        /**
         * Handler for switchSet command
         * @param callback
         */
        this.onSwitchSet = (_value, callback) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            // Execute sequence
            for (const step of this._sequence.Steps) {
                yield util_1.sleep(step.Delay);
                const device = this._devices[(_a = step.DeviceName) !== null && _a !== void 0 ? _a : ""];
                if (device &&
                    step.DeviceCommand &&
                    device.supportsCommand(step.DeviceCommand)) {
                    yield this._dataProvider.sendCommand(step.DeviceCommand, device);
                }
                else {
                    this._platform.log.warn(`Attempted to execute command ${step.DeviceCommand} on device ${step.DeviceName} but the device or command was not found`);
                }
            }
            // Deactivate button
            this._switchService
                .getCharacteristic(this._platform.Characteristic.On)
                .updateValue(false);
            callback(null);
        });
        this._accessory
            .getService(this._platform.Service.AccessoryInformation)
            .setCharacteristic(this._platform.Characteristic.Manufacturer, "Brandon Watson")
            .setCharacteristic(this._platform.Characteristic.Model, "Sequence Button")
            .setCharacteristic(this._platform.Characteristic.SerialNumber, "123-456-789");
        const switchUUID = this._platform.api.hap.uuid.generate(`${this._accessory.displayName} Switch`);
        this._switchService =
            this._accessory.getService(this._platform.Service.Switch) ||
                this._accessory.addService(this._platform.Service.Switch, this._accessory.displayName, switchUUID);
        this._switchService
            .getCharacteristic(this._platform.Characteristic.On)
            .on("set", this.onSwitchSet)
            .updateValue(false)
            .on("get", (callback) => {
            return callback(null);
        });
        this._devices = {};
        // Get devices in sequence
        for (const deviceName of _sequence.Steps.map((e) => e.DeviceName)) {
            if (!deviceName) {
                continue;
            }
            const device = this._dataProvider.getDeviceByName(deviceName);
            if (device) {
                this._devices[deviceName] = device;
            }
            else {
                this._platform.log.warn(`Device ${deviceName} was not found in harmony configuration`);
            }
        }
    }
}
exports.Sequence = Sequence;
//# sourceMappingURL=sequence.js.map