"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HarmonyDataProvider = void 0;
const tsyringe_1 = require("tsyringe");
const harmonyHub_1 = require("../models/harmonyHub");
let HarmonyDataProvider = class HarmonyDataProvider {
    constructor(_config, _log) {
        this._config = _config;
        this._log = _log;
        this._hubs = {};
        this._deviceConfigByName = {};
        this.connect = (hubs) => __awaiter(this, void 0, void 0, function* () {
            let readyCount = 0;
            yield Promise.all(hubs.map((hub) => __awaiter(this, void 0, void 0, function* () {
                const newHarmonyHub = new harmonyHub_1.HarmonyHub(hub.Name, hub.Ip, this._log.info);
                this._hubs[hub.Name] = newHarmonyHub;
                newHarmonyHub.on("Ready", () => {
                    readyCount++;
                    if (readyCount === Object.keys(this._hubs).length) {
                        // this.emit("Ready");
                    }
                });
                yield newHarmonyHub.initialize();
            })));
        });
        _config.Devices.forEach((deviceConfig) => {
            this._deviceConfigByName[deviceConfig.Name] = deviceConfig;
        });
        this.connect(_config.Hubs);
        this.emitInfo();
    }
    powerOnDevices(devices) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(devices.map((device) => __awaiter(this, void 0, void 0, function* () {
                if (device && device.name) {
                    if (!device.on) {
                        this._log.info(`Turning on device ${device.name}`);
                        yield this.powerOnDevice(device);
                    }
                }
            })));
        });
    }
    powerOffDevices(devices) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(
            //Turn off devices
            devices.map((device) => __awaiter(this, void 0, void 0, function* () {
                if (device) {
                    if (device.on) {
                        this._log.info(`Turning off device ${device.name}`);
                        yield this.powerOffDevice(device);
                    }
                }
            })));
        });
    }
    sendCommand(commandName, harmonyDevice) {
        return __awaiter(this, void 0, void 0, function* () {
            let command;
            commandName = this.getOverrideCommand(commandName, harmonyDevice);
            if (harmonyDevice.supportsCommand(commandName)) {
                command = harmonyDevice.getCommand(commandName);
            }
            const hub = this.getHubByDevice(harmonyDevice);
            yield hub.sendCommand(command);
        });
    }
    /**
     * Get the IDevice by name.
     * @param deviceName The device to retrieve.
     */
    getDeviceByName(deviceName) {
        let device;
        try {
            device =
                this._hubs[this._deviceConfigByName[deviceName].Hub].getDeviceByName(deviceName);
        }
        catch (err) {
            this._log.info(`Error retrieving device from hub: ${err}`);
        }
        return device;
    }
    emitInfo() {
        //Emit devices if requested
        this._log.info("All hubs connected");
        if (this._config.EmitDevicesOnStartup) {
            const hubs = this._hubs;
            Object.values(hubs).forEach((hub) => {
                const deviceDictionary = hub.devices;
                this._log.info(`${hub.hubName}`);
                Object.values(deviceDictionary).forEach((device) => {
                    this._log.info(`    ${device.name} : ${device.id}`);
                    Object.keys(device.commands).forEach((command) => {
                        this._log.info(`        ${command}`);
                    });
                });
            });
        }
    }
    getHubByDevice(device) {
        return this._hubs[this._deviceConfigByName[device.name].Hub];
    }
    powerOnDevice(harmonyDevice) {
        return __awaiter(this, void 0, void 0, function* () {
            let powerOnCommand = "Power On";
            let powerToggleCommand = "Power Toggle";
            if (harmonyDevice.supportsCommand(powerOnCommand)) {
                yield this.sendCommand(powerOnCommand, harmonyDevice);
                harmonyDevice.on = true;
            }
            else if (harmonyDevice.supportsCommand(powerToggleCommand)) {
                yield this.sendCommand(powerToggleCommand, harmonyDevice);
                harmonyDevice.on = true;
            }
            else {
                yield this.sendCommand(powerOnCommand, harmonyDevice);
            }
        });
    }
    powerOffDevice(harmonyDevice) {
        return __awaiter(this, void 0, void 0, function* () {
            let powerOffCommand = "Power Off";
            let powerToggleCommand = "Power Toggle";
            if (harmonyDevice.supportsCommand(powerOffCommand)) {
                yield this.sendCommand(powerOffCommand, harmonyDevice);
                harmonyDevice.on = false;
            }
            else if (harmonyDevice.supportsCommand(powerToggleCommand)) {
                yield this.sendCommand(powerToggleCommand, harmonyDevice);
                harmonyDevice.on = false;
            }
        });
    }
    getOverrideCommand(commandName, harmonyDevice) {
        const deviceConfig = this._deviceConfigByName[harmonyDevice.name];
        if (!deviceConfig.Overrides) {
            return commandName;
        }
        const overrideCommand = deviceConfig.Overrides.find((e) => e.Command == commandName);
        return overrideCommand ? overrideCommand.Override : commandName;
    }
};
HarmonyDataProvider = __decorate([
    tsyringe_1.injectable(),
    __param(0, tsyringe_1.inject("IConfig")),
    __param(1, tsyringe_1.inject("log")),
    __metadata("design:paramtypes", [Object, Function])
], HarmonyDataProvider);
exports.HarmonyDataProvider = HarmonyDataProvider;
//# sourceMappingURL=harmonyDataProvider.js.map