"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HarmonyHub = void 0;
const harmonyDevice_1 = require("./harmonyDevice");
const Harmony = require("harmony-websocket");
const events_1 = require("events");
const util_1 = require("../util");
class HarmonyHub extends events_1.EventEmitter {
    constructor(hubName, ipAddress, log) {
        super();
        this._devices = {};
        this.getDeviceByName = (deviceName) => {
            return this._devices[deviceName];
        };
        this.initialize = () => __awaiter(this, void 0, void 0, function* () {
            this._harmony = new Harmony();
            yield this._harmony.connect(this._ip);
            this._harmony.on("stateDigest", (data) => {
                console.log(data);
            });
            this._harmony.on("automationState", (data) => {
                console.log(data);
            });
            //Gather devices
            let devices = yield this._harmony.getDevices();
            try {
                yield Promise.all(
                //Add each to dictionary
                devices.map((dev) => __awaiter(this, void 0, void 0, function* () {
                    //get commands
                    let commands = {};
                    let deviceCommands = yield this._harmony.getDeviceCommands(dev.id);
                    deviceCommands.forEach((command) => {
                        commands[command.label] = command.action;
                    });
                    this._devices[dev.label] = new harmonyDevice_1.HarmonyDevice({
                        id: dev.id,
                        name: dev.label,
                        commands: commands,
                        log: this._log,
                        harmony: this._harmony,
                    });
                })));
                this.emit("Ready");
            }
            catch (err) {
                this._log(`ERROR - error connecting to harmony: ${err}`);
            }
        });
        this.connect = () => __awaiter(this, void 0, void 0, function* () {
            yield this._harmony.Connect(this._ip);
        });
        this._ip = ipAddress;
        this._log = log;
        this._name = hubName;
    }
    get devices() {
        return this._devices;
    }
    get hubName() {
        return this._name;
    }
    sendCommand(command) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                //Execute command
                //HACK to fix Harmon Kardon receiver not turning off
                if (command.command === "PowerOff") {
                    for (let i = 0; i < 2; i++) {
                        yield this._harmony.sendCommand(JSON.stringify(command));
                    }
                }
                yield this._harmony.sendCommand(JSON.stringify(command));
                //Sleep
                yield util_1.sleep(800);
            }
            catch (err) {
                this._log(`ERROR - error sending command to harmony: ${err}`);
            }
        });
    }
}
exports.HarmonyHub = HarmonyHub;
//# sourceMappingURL=harmonyHub.js.map