import { Logging } from "homebridge";
import { HarmonyDataProvider } from "../dataProviders/harmonyDataProvider";
import { StateDataProvider } from "../dataProviders/stateDataProvider";
import { IActivity, IConfig } from "../models/config";
export declare class ActivityService {
    private _harmonyDataProvider;
    private _stateDataProvider;
    private _config;
    private _log;
    constructor(_harmonyDataProvider: HarmonyDataProvider, _stateDataProvider: StateDataProvider, _config: IConfig, _log: Logging);
    startLastOrDefaultActivity: (controlUnitName: string, defaultActivity: IActivity) => Promise<void>;
    startActivity: (controlUnitName: string, activity: IActivity) => Promise<void>;
    stopCurrentActivity: (controlUnitName: string) => Promise<void>;
    getCurrentActivity(controlUnitName: string): IActivity | undefined;
    getLastActivity(controlUnitName: string): IActivity | undefined;
    /**
     * Return if a control unit is active
     * @param controlUnitName
     */
    getIsActive(controlUnitName: string): boolean;
    /**
     * Helper function to make sure no control unit depends on device list.
     * @param devicesToTurnOn The list of devices to modify.
     * @param controlUnitName The name of the control unit in question.
     */
    private sanitizeDeviceList;
    private getDevicesToTurnOn;
    private getDevicesToTurnOff;
    private buildPotentialDeviceList;
    private assignDeviceInput;
    private routeInputsToOutputs;
}
