"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActivityService = void 0;
const tsyringe_1 = require("tsyringe");
const harmonyDataProvider_1 = require("../dataProviders/harmonyDataProvider");
const stateDataProvider_1 = require("../dataProviders/stateDataProvider");
let ActivityService = class ActivityService {
    constructor(_harmonyDataProvider, _stateDataProvider, _config, _log) {
        this._harmonyDataProvider = _harmonyDataProvider;
        this._stateDataProvider = _stateDataProvider;
        this._config = _config;
        this._log = _log;
        this.startLastOrDefaultActivity = (controlUnitName, defaultActivity) => __awaiter(this, void 0, void 0, function* () {
            const lastActivity = this.getLastActivity(controlUnitName);
            if (!lastActivity) {
                return this.startActivity(controlUnitName, defaultActivity);
            }
            return this.startActivity(controlUnitName, lastActivity);
        });
        this.startActivity = (controlUnitName, activity) => __awaiter(this, void 0, void 0, function* () {
            this._log.info(`Starting activity ${activity.DisplayName} for controlUnit: ${controlUnitName}`);
            let devicesToTurnOn = this.getDevicesToTurnOn(activity, controlUnitName);
            yield this._harmonyDataProvider.powerOnDevices(devicesToTurnOn);
            yield this.assignDeviceInput(activity);
            yield this.routeInputsToOutputs(activity);
            let lastActivity = this.getCurrentActivity(controlUnitName);
            let devicesToTurnOff = this.getDevicesToTurnOff(controlUnitName, lastActivity, activity);
            yield this._harmonyDataProvider.powerOffDevices(devicesToTurnOff);
            this._stateDataProvider.updateState(activity, controlUnitName);
        });
        this.stopCurrentActivity = (controlUnitName) => __awaiter(this, void 0, void 0, function* () {
            if (!this.getCurrentActivity(controlUnitName)) {
                return;
            }
            let lastActivity = this.getCurrentActivity(controlUnitName);
            let devicesToTurnOff = this.getDevicesToTurnOff(controlUnitName, lastActivity);
            yield this._harmonyDataProvider.powerOffDevices(devicesToTurnOff);
            this._stateDataProvider.deactivateState(controlUnitName);
        });
    }
    getCurrentActivity(controlUnitName) {
        var _a;
        return (_a = this._stateDataProvider.getState(controlUnitName)) === null || _a === void 0 ? void 0 : _a.currentActivity;
    }
    getLastActivity(controlUnitName) {
        var _a;
        return (_a = this._stateDataProvider.getState(controlUnitName)) === null || _a === void 0 ? void 0 : _a.lastActivity;
    }
    /**
     * Return if a control unit is active
     * @param controlUnitName
     */
    getIsActive(controlUnitName) {
        const state = this._stateDataProvider.getState(controlUnitName);
        return state != undefined && state.currentActivity != undefined;
    }
    /**
     * Helper function to make sure no control unit depends on device list.
     * @param devicesToTurnOn The list of devices to modify.
     * @param controlUnitName The name of the control unit in question.
     */
    sanitizeDeviceList(devicesToTurnOn, controlUnitName) {
        for (let controlUnitKey in this._stateDataProvider.states) {
            //Skip self
            if (controlUnitKey === controlUnitName) {
                continue;
            }
            let currentOtherState = this._stateDataProvider.states[controlUnitKey];
            if (!currentOtherState || !currentOtherState.currentActivity) {
                return devicesToTurnOn;
            }
            currentOtherState.currentActivity.DeviceSetupList.forEach((value) => {
                //there are devices to remove
                if (devicesToTurnOn.some((e) => e && e.name === value.DeviceName)) {
                    let deviceToRemove = devicesToTurnOn.filter((i) => i.name === value.DeviceName)[0];
                    delete devicesToTurnOn[devicesToTurnOn.indexOf(deviceToRemove)];
                }
            });
        }
        return devicesToTurnOn;
    }
    getDevicesToTurnOn(activity, controlUnitName) {
        let potentialDevices = this.buildPotentialDeviceList(activity);
        return this.sanitizeDeviceList(potentialDevices, controlUnitName);
    }
    getDevicesToTurnOff(controlUnitName, lastActivity, nextActivity) {
        let potentialDevices = lastActivity
            ? this.buildPotentialDeviceList(lastActivity)
            : [];
        //remove devices that will be used for next activity from list
        //delete array[index] is stupid because it just nulls out the index. But now i have to deal with nulls
        if (nextActivity) {
            potentialDevices.forEach((device, index) => {
                if (device &&
                    device.name &&
                    nextActivity.DeviceSetupList.some((e) => {
                        return e && e.DeviceName === device.name;
                    })) {
                    delete potentialDevices[index];
                }
            });
        }
        return this.sanitizeDeviceList(potentialDevices, controlUnitName);
    }
    buildPotentialDeviceList(activity) {
        return activity.DeviceSetupList.map((value) => {
            return this._harmonyDataProvider.getDeviceByName(value.DeviceName);
        });
    }
    assignDeviceInput(activity) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(activity.DeviceSetupList.map((value) => __awaiter(this, void 0, void 0, function* () {
                let device = this._harmonyDataProvider.getDeviceByName(value.DeviceName);
                if (device && device.supportsCommand(`Input${value.Input}`)) {
                    yield this._harmonyDataProvider.sendCommand(`Input${value.Input}`, device);
                }
            })));
        });
    }
    routeInputsToOutputs(activity) {
        return __awaiter(this, void 0, void 0, function* () {
            if (activity.UseMatrix) {
                //get input and output
                let input = this._config.Matrix.Inputs.filter((e) => e.InputDevice === activity.ControlDevice)[0];
                let output = this._config.Matrix.Outputs.filter((e) => e.OutputDevice === activity.OutputDevice)[0];
                let inputCommandName = `In ${input.InputNumber}`;
                let outputCommandName = `Out ${output.OutputLetter}`;
                let matrixDevice = this._harmonyDataProvider.getDeviceByName(this._config.Matrix.DeviceName);
                //Route hdmi
                if (matrixDevice.supportsCommand(inputCommandName) &&
                    matrixDevice.supportsCommand(outputCommandName)) {
                    yield this._harmonyDataProvider.sendCommand(outputCommandName, matrixDevice);
                    yield this._harmonyDataProvider.sendCommand(inputCommandName, matrixDevice);
                    yield this._harmonyDataProvider.sendCommand(outputCommandName, matrixDevice);
                    yield this._harmonyDataProvider.sendCommand(inputCommandName, matrixDevice);
                }
            }
        });
    }
};
ActivityService = __decorate([
    tsyringe_1.injectable(),
    __param(0, tsyringe_1.inject(harmonyDataProvider_1.HarmonyDataProvider)),
    __param(1, tsyringe_1.inject(stateDataProvider_1.StateDataProvider)),
    __param(2, tsyringe_1.inject("IConfig")),
    __param(3, tsyringe_1.inject("log")),
    __metadata("design:paramtypes", [harmonyDataProvider_1.HarmonyDataProvider,
        stateDataProvider_1.StateDataProvider, Object, Function])
], ActivityService);
exports.ActivityService = ActivityService;
//# sourceMappingURL=activityService.js.map