"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandService = void 0;
const tsyringe_1 = require("tsyringe");
const controlUnit_1 = require("../accessories/controlUnit");
const harmonyDataProvider_1 = require("../dataProviders/harmonyDataProvider");
const stateDataProvider_1 = require("../dataProviders/stateDataProvider");
let CommandService = class CommandService {
    constructor(_stateDataProvider, _harmonyDataProvider) {
        this._stateDataProvider = _stateDataProvider;
        this._harmonyDataProvider = _harmonyDataProvider;
        /**
         * Send key press for current activity.
         *
         * @param controlUnitName The name of the control unit to act on.
         * @param key The key to send.
         */
        this.sendKeyPress = (controlUnitName, key) => __awaiter(this, void 0, void 0, function* () {
            let currentActivity = this._stateDataProvider.getState(controlUnitName);
            if (currentActivity) {
                let commandName = "";
                let device = this._harmonyDataProvider.getDeviceByName(currentActivity.currentActivity.ControlDevice);
                switch (key) {
                    case controlUnit_1.RemoteKey.ARROW_UP: {
                        commandName = "Direction Up";
                        break;
                    }
                    case controlUnit_1.RemoteKey.ARROW_DOWN: {
                        commandName = "Direction Down";
                        break;
                    }
                    case controlUnit_1.RemoteKey.ARROW_LEFT: {
                        commandName = "Direction Left";
                        break;
                    }
                    case controlUnit_1.RemoteKey.ARROW_RIGHT: {
                        commandName = "Direction Right";
                        break;
                    }
                    case controlUnit_1.RemoteKey.SELECT: {
                        commandName = "Select";
                        break;
                    }
                    case controlUnit_1.RemoteKey.PLAY_PAUSE: {
                        commandName = "Pause";
                        break;
                    }
                    case controlUnit_1.RemoteKey.INFORMATION: {
                        commandName = "Menu";
                        break;
                    }
                    case controlUnit_1.RemoteKey.BACK: {
                        commandName = "Back";
                        break;
                    }
                    case controlUnit_1.RemoteKey.EXIT: {
                        commandName = "Back";
                        break;
                    }
                }
                yield this._harmonyDataProvider.sendCommand(commandName, device);
            }
        });
    }
};
CommandService = __decorate([
    tsyringe_1.injectable(),
    __param(0, tsyringe_1.inject(stateDataProvider_1.StateDataProvider)),
    __param(1, tsyringe_1.inject(harmonyDataProvider_1.HarmonyDataProvider)),
    __metadata("design:paramtypes", [stateDataProvider_1.StateDataProvider,
        harmonyDataProvider_1.HarmonyDataProvider])
], CommandService);
exports.CommandService = CommandService;
//# sourceMappingURL=commandService.js.map